/**
  Generated Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This is the main file generated using PIC10 / PIC12 / PIC16 / PIC18 MCUs

  Description:
    This header file provides implementations for driver APIs for all modules selected in the GUI.
    Generation Information :
        Product Revision  :  PIC10 / PIC12 / PIC16 / PIC18 MCUs - 1.81.5
        Device            :  PIC16F18877
        Driver Version    :  2.00
*/

/*
    (c) 2018 Microchip Technology Inc. and its subsidiaries. 
    
    Subject to your compliance with these terms, you may use Microchip software and any 
    derivatives exclusively with Microchip products. It is your responsibility to comply with third party 
    license terms applicable to your use of third party software (including open source software) that 
    may accompany Microchip software.
    
    THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
    EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY 
    IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS 
    FOR A PARTICULAR PURPOSE.
    
    IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
    WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP 
    HAS BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO 
    THE FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL 
    CLAIMS IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT 
    OF FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS 
    SOFTWARE.
*/

// button definitions

#define button1  RB1     // button B1 is connected to RB1 pin
#define button2  RB2   // button B2 is connected to RB2 pin
#define button3  RB3   // button B2 is connected to RB3 pin
#define LED_PIN  RB4   // Alarm LED is connected to RB4 pin


#include "mcc_generated_files/mcc.h"
#include <xc.h> 
#include <stdint.h>

void Initialize(void)
{
    
 SYSTEM_Initialize();    // initialize the device

}

char Time[]     = "  :  :  ",
     Date[] = "  /  /20  ",
     Alarm1[]   = "A1:   :  :00", Alarm2[]   = "A2:   :  :00",
     Temperature[] = "T:   .   C";
uint8_t  i, second, minute, hour, w_day, m_day, month, year,
         alarm1_minute, alarm1_hour, alarm2_minute, alarm2_hour,
         status_reg, alarm1_status, alarm2_status;

// external interrupt ISR
void __interrupt() EXT(void)
{
  if (INTF){
    LED_PIN = 1;
    INTF = 0;
  }
}

/*********************************** I2C functions **************************/

void I2C_Init(uint32_t i2c_clk_freq)
{
    SSP1CON1  = 0x28;  // configure MSSP module to work in I2C mode
    SSP1ADD  = (_XTAL_FREQ /(4 * i2c_clk_freq)) - 1;  // set I2C clock frequency
    SSP1STAT = 0;
}
   
void I2C_Start()
{
    while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));  // wait for MSSP module to be free (not busy)
    SEN1 = 1;  // initiate start condition
}

void I2C_Repeated_Start()
{
  while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));  // wait for MSSP module to be free (not busy)
  RSEN1 = 1;  // initiate repeated start condition
}

void I2C_Stop()
{
  while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));  // wait for MSSP module to be free (not busy)
  PEN1 = 1;  // initiate stop condition
}

void I2C_Write(uint8_t i2c_data)
{
  while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));  // wait for MSSP module to be free (not busy)
  SSP1BUF = i2c_data;  // update buffer
}

uint8_t I2C_Read(uint8_t ack)
{
  uint8_t _data;
  while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));  // wait for MSSP module to be free (not busy)
  RCEN1 = 1;
  while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));  // wait for MSSP module to be free (not busy)
  _data = SSP1BUF;  // read data from buffer
  while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));  // wait for MSSP module to be free (not busy)
  // send acknowledge pulse ? (depends on ack, if 1 send, otherwise don't send)
  ACKDT1 = !ack;
  ACKEN1 = 1;
  return _data;  // return data read
}
/******************************************** end I2C functions **********************/
 
// function for button debounce
__bit debounce (uint8_t button)
{
  uint8_t count = 0;
  for(uint8_t i = 0; i < 5; i++)
  {
    switch (button)
    {
      case 1: if (button1 == 0)
                count++;
                break;
      case 3: if (button3 == 0)
                count++;
                break;
      default:  break;
    }
    __delay_ms(10);
  }
  if(count > 2)  return 1;
  else           return 0;
}

/********************** RTC chip functions *********************/
// convert BCD to decimal function
uint8_t bcd_to_decimal(uint8_t number)
{
  return((number >> 4) * 10 + (number & 0x0F));
}

// convert decimal to BCD function
uint8_t decimal_to_bcd(uint8_t number)
{
  return(((number / 10) << 4) + (number % 10));
}

void RTC_read()    // read current time & date from RTC chip
{
  I2C_Start();           // start I2C
  I2C_Write(0xD0);       // RTC chip address
  I2C_Write(0);          // send register address
  I2C_Repeated_Start();  // restart I2C
  I2C_Write(0xD1);       // initialize data read
  second = I2C_Read(1);  // read seconds from register 0
  minute = I2C_Read(1);  // read minutes from register 1
  hour   = I2C_Read(1);  // read hour from register 2
  w_day  = I2C_Read(1);  // read day of the week from register 3
  m_day  = I2C_Read(1);  // read date from register 4
  month  = I2C_Read(1);  // read month from register 5
  year   = I2C_Read(0);  // read year from register 6
  I2C_Stop();            // stop I2C
}

void alarms_read_display()   // read and display alarm1 and alarm2 data function
{
  uint8_t control_reg, temperature_lsb;
  int8_t temperature_msb;
  I2C_Start();          // start I2C
  I2C_Write(0xD0);      // RTC chip address
  I2C_Write(0x08);      // send register address (alarm1 minutes register)
  I2C_Repeated_Start(); // restart I2C
  I2C_Write(0xD1);      // initialize data read
  alarm1_minute = I2C_Read(1);   // read alarm1 minutes
  alarm1_hour   = I2C_Read(1);   // read alarm1 hours
  I2C_Read(1);                   // skip alarm1 day/date register
  alarm2_minute = I2C_Read(1);   // read alarm2 minutes
  alarm2_hour   = I2C_Read(1);   // read alarm2 hours
  I2C_Read(1);                   // skip alarm2 day/date register
  control_reg = I2C_Read(1);     // read control register
  status_reg  = I2C_Read(1);     // read status register
  I2C_Read(1);                   // skip aging offset register
  temperature_msb = I2C_Read(1); // read temperature MSB
  temperature_lsb = I2C_Read(0); // read temperature LSB
  I2C_Stop();                    // stop I2C
  // convert BCD to decimal
  alarm1_minute = bcd_to_decimal(alarm1_minute);
  alarm1_hour   = bcd_to_decimal(alarm1_hour);
  alarm2_minute = bcd_to_decimal(alarm2_minute);
  alarm2_hour   = bcd_to_decimal(alarm2_hour);
  // end conversion
  // update Alarm1 and Alarm2
  Alarm1[8]     = alarm1_minute % 10  + '0';
  Alarm1[7]     = alarm1_minute / 10  + '0';
  Alarm1[5]     = alarm1_hour   % 10  + '0';
  Alarm1[4]     = alarm1_hour   / 10  + '0';
  Alarm2[8]     = alarm2_minute % 10  + '0';
  Alarm2[7]     = alarm2_minute / 10  + '0';
  Alarm2[5]     = alarm2_hour   % 10  + '0';
  Alarm2[4]     = alarm2_hour   / 10  + '0';
  alarm1_status = control_reg & 0x01;      // read alarm1 interrupt enable bit (A1IE) from RTC chip control register
  alarm2_status = (control_reg >> 1) & 0x01; // read alarm2 interrupt enable bit (A2IE) from RTC chip control register
  if(temperature_msb < 0){
    temperature_msb *= -1;
    Temperature[2] = '-';
  }
  else
    Temperature[2] = ' ';
  temperature_lsb >>= 6;
  Temperature[4] = temperature_msb % 10  + '0';
  Temperature[3] = temperature_msb / 10  + '0';
  if(temperature_lsb == 0 || temperature_lsb == 2){
    Temperature[7] = '0';
    if(temperature_lsb == 0) Temperature[6] = '0';
    else                     Temperature[6] = '5';
  }
  if(temperature_lsb == 1 || temperature_lsb == 3){
    Temperature[7] = '5';
    if(temperature_lsb == 1) Temperature[6] = '2';
    else                     Temperature[6] = '7';
  }
  Temperature[8]  = 223;   // put degree symbol (°)
  LCD_Goto(11, 1);
  LCD_Print(Temperature);  // print temperature
  LCD_Goto(1, 3);
  LCD_Print(Alarm1);       // print Alarm1
  if(alarm1_status) {
    LCD_Goto(18, 3);
    LCD_Print("ON ");      // if A1IE = 1 print 'ON'
  }
  else {
    LCD_Goto(18, 3);
    LCD_Print("OFF");   // if A1IE = 0 print 'OFF'
  }
  LCD_Goto(1, 4);
  LCD_Print(Alarm2);    // print Alarm2
  if(alarm2_status){
    LCD_Goto(18, 4);
    LCD_Print("ON ");   // if A2IE = 1 print 'ON'
  }
  else {
    LCD_Goto(18, 4);
    LCD_Print("OFF");   // if A2IE = 0 print 'OFF'
  }
}

void w_day_display()    // print day of the week
{
  LCD_Goto(1, 2);
  switch(w_day){
    case 1:  LCD_Print("Sun"); break;
    case 2:  LCD_Print("Mon"); break;
    case 3:  LCD_Print("Tue"); break;
    case 4:  LCD_Print("Wed"); break;
    case 5:  LCD_Print("Thu"); break;
    case 6:  LCD_Print("Fri"); break;
    default: LCD_Print("Sat");
  }
}

// display time and date function
void RTC_display()
{
  // convert data from BCD format to decimal format
  second = bcd_to_decimal(second);
  minute = bcd_to_decimal(minute);
  hour   = bcd_to_decimal(hour);
  m_day  = bcd_to_decimal(m_day);
  month  = bcd_to_decimal(month);
  year   = bcd_to_decimal(year);
  // end conversion
  // update time
  Time[7] = second % 10  + '0';
  Time[6] = second / 10  + '0';
  Time[4] = minute % 10  + '0';
  Time[3] = minute / 10  + '0';
  Time[1] = hour   % 10  + '0';
  Time[0] = hour   / 10  + '0';
  // update date
  Date[9] = year  % 10 + '0';
  Date[8] = year  / 10 + '0';
  Date[4] = month % 10 + '0';
  Date[3] = month / 10 + '0';
  Date[1] = m_day  % 10 + '0';
  Date[0] = m_day  / 10 + '0';

  LCD_Goto(1, 1);    // go to column 1, row 1
  LCD_Print(Time);   // print time
  LCD_Goto(5, 2);    // go to column 1, row 2
  LCD_Print(Date);   // print date
  w_day_display();   // print day of the week

}

// make editing parameter blinks function
void blink()
{
  uint8_t j = 0;
  while( j < 100 && (button1 || i >= 5) && button2 && (button3 || i < 5) )
  {
    j++;
    __delay_ms(5);
  }
}

// edit time and date function
uint8_t edit(uint8_t x, uint8_t y, uint8_t parameter)
{
  while(debounce(1) || debounce(3));  // call debounce function (wait for B1 & B3 to be released)

  while(1) {

    while(!button2)    // if button B2 is pressed
    {
      parameter++;
      if(((i == 0) || (i == 5)) && parameter > 23)   // if hours > 23 ==> hours = 0
        parameter = 0;
      if(((i == 1) || (i == 6)) && parameter > 59)   // if minutes > 59 ==> minutes = 0
        parameter = 0;
      if(i == 2 && parameter > 31)   // if date > 31 ==> date = 1
        parameter = 1;
      if(i == 3 && parameter > 12)   // if month > 12 ==> month = 1
        parameter = 1;
      if(i == 4 && parameter > 99)   // if year > 99 ==> year = 0
        parameter = 0;
      if(i == 7 && parameter > 1)    // for alarms ON or OFF (1: alarm ON, 0: alarm OFF)
        parameter = 0;

      LCD_Goto(x, y);
      LCD_PutC(parameter / 10 + '0');
      LCD_PutC(parameter % 10 + '0');

      if(i >= 5){
        RTC_read();     // read data from RTC chip
        RTC_display();  // display time and date
      }
      __delay_ms(200);
    }

    LCD_Goto(x, y);
    LCD_Print("  ");  // print 2 spaces
    blink();

    LCD_Goto(x, y);
    LCD_PutC(parameter / 10 + '0');
    LCD_PutC(parameter % 10 + '0');
    blink();

    if(i >= 5)
    {
      RTC_read();
      RTC_display();
    }

    if((!button1 && i < 5) || (!button3 && i >= 5))     // if B1 or B3 is pressed
    {
      i++;   // increment 'i' for the next parameter
      return parameter;     // return parameter value and exit
    }
  }
}

uint8_t on_off(uint8_t x, uint8_t y, uint8_t al)
{
  while(debounce(3));  // call debounce function (wait for B3 to be released)
  while (1)
  {
    while(!button2)    // if button B2 is pressed
    {
      al = (!al) & 0x01;
      LCD_Goto(x, y);
      if(al == 1)  LCD_Print("ON ");
      else         LCD_Print("OFF");
      __delay_ms(200);
    }

    LCD_Goto(x, y);
    LCD_Print("   ");  // print 3 spaces
    blink();

    LCD_Goto(x, y);
    if(al == 1)  LCD_Print("ON ");
    else         LCD_Print("OFF");
    blink();

    RTC_read();
    RTC_display();

    if(!button3)
    if(debounce(3))
    {
     return al; 
    }
  }
}

/********************** end RTC chip functions *****************/

/*************************** main function *********************/
void main(void)
{
  OSCFRQ = 0x03;   // set internal oscillator to 8MHz
  ANSELB = 0;      // configure all PORTB pins as digital
  PORTB  = 0;      // PORTB initial state
  TRISB  = 0x0F;   // configure RB0 ~ 3 as input pins
  // enable RB0, RB1, RB2 and RB3 internal pull ups
  //  = 0;      // clear RBPU bit (OPTION_REG.7)
  WPUB   = 0x0F;   // WPUB register = 0b00001111

  INTE   = 0;      // clear external interrupt flag bit
  GIE    = 1;      // enable global interrupts
  INTEDG = 0;      // set external Interrupt on falling edge
  INTE   = 1;      // enable external interrupt

  __delay_ms(1000);  // wait 1 second

  I2C_Init(100000);   // initialize I2C bus with clock frequency of 100kHz

  LCD_Begin();        // initialize LCD module

  while(1) {

    if(!button1)     // if button B1 is pressed
    if(debounce(1))   // call debounce function (make sure if B1 is pressed)
    {
      i = 0;
      hour   = edit(1, 1, hour);
      minute = edit(4, 1, minute);

      while(debounce(1));  // call debounce function (wait for button B1 to be released)

      while(1)
      {
        while(!button2)    // if button RB2 button is pressed
        {
          w_day++;    // increment day
          if(w_day > 7) w_day = 1;
          w_day_display();
          __delay_ms(200);
        }

        LCD_Goto(1, 2);
        LCD_Print("   ");  // print 3 spaces
        blink();           // call blink function
        w_day_display();
        blink();           // call blink function
        if(!button1)       // if button B1 is pressed
        if(debounce(1))    // call debounce function (make sure if B1 is pressed)
          break;
      }

      m_day  = edit(5,  2, m_day);
      month  = edit(8,  2, month);
      year   = edit(13, 2, year);

      while(debounce(1));  // call debounce function (wait for button B1 to be released)

      // convert decimal to BCD
      minute = decimal_to_bcd(minute);
      hour   = decimal_to_bcd(hour);
      m_day  = decimal_to_bcd(m_day);
      month  = decimal_to_bcd(month);
      year   = decimal_to_bcd(year);
      // end conversion

      // write data to DS3231 RTC
      I2C_Start();         // start I2C
      I2C_Write(0xD0);     // RTC chip address
      I2C_Write(0);        // send register address
      I2C_Write(0);        // reset seconds and start oscillator
      I2C_Write(minute);   // write minute value to RTC chip
      I2C_Write(hour);     // write hour value to RTC chip
      I2C_Write(w_day);    // write day of the week value to RTC chip
      I2C_Write(m_day);    // write date (day of the month) value to RTC chip
      I2C_Write(month);    // write month value to RTC chip
      I2C_Write(year);     // write year value to RTC chip
      I2C_Stop();          // stop I2C

      __delay_ms(200);

    }

    if(!button3)     // if RB3 button is pressed
    if(debounce(3))  // call debounce function (make sure if B3 is pressed)
    {  
      i = 5;
      alarm1_hour   = edit(5, 3, alarm1_hour);
      alarm1_minute = edit(8, 3, alarm1_minute);
      alarm1_status = on_off(18, 3, alarm1_status);
      i = 5;
      alarm2_hour   = edit(5, 4, alarm2_hour);
      alarm2_minute = edit(8, 4, alarm2_minute);
      alarm2_status = on_off(18, 4, alarm2_status);

      while(debounce(3));  // call debounce function (wait for button B3 to be released)
      // convert decimal to BCD
      alarm1_minute = decimal_to_bcd(alarm1_minute);
      alarm1_hour   = decimal_to_bcd(alarm1_hour);
      alarm2_minute = decimal_to_bcd(alarm2_minute);
      alarm2_hour   = decimal_to_bcd(alarm2_hour);
      // end conversion
      // write alarms data to DS3231
      I2C_Start();      // start I2C
      I2C_Write(0xD0);  // RTC chip address
      I2C_Write(7);     // send register address (alarm1 seconds)
      I2C_Write(0);     // write 0 to alarm1 seconds
      I2C_Write(alarm1_minute);   // write alarm1 minutes
      I2C_Write(alarm1_hour);     // write alarm1 hours
      I2C_Write(0x80);            // alarm1 when hours, minutes, and seconds match
      I2C_Write(alarm2_minute);   // write alarm2 minutes
      I2C_Write(alarm2_hour);     // write alarm2 hours
      I2C_Write(0x80);            // alarm2 when hours and minutes match
      I2C_Write(4 | alarm1_status | (alarm2_status << 1));    // write data to control register (enable interrupt when alarm)
      I2C_Write(0);     // clear alarm flag bits
      I2C_Stop();       // stop I2C
      __delay_ms(200);  // wait 200ms
    }
    if(!button2 && LED_PIN){   // when button B2 pressed with alarm (Reset and turn OFF the alarm)
      LED_PIN = 0;             // turn OFF alarm indicator
      I2C_Start();             // start I2C
      I2C_Write(0xD0);         // RTC chip address
      I2C_Write(0x0E);         // send register address (control register)
      // write data to control register (Turn OFF the occurred alarm and keep the other as it is)
      I2C_Write(4 | ((!(status_reg & 1)) & alarm1_status) | (((!((status_reg >> 1) & 1)) & alarm2_status) << 1));
      I2C_Write(0);    // clear alarm flag bits
      I2C_Stop();      // stop I2C
    }

    RTC_read();             // read time and date from RTC chip
    alarms_read_display();  // read and display alarms
    RTC_display();          // print time & date

    __delay_ms(50);   // wait 50 ms

  }

}

/*************************** end main function ********************************/



void ToggleRA0Led()
{
while (1)

{ 
    IO_RA0_Toggle();    // Add your application code
    __delay_ms(1000);
}    
    
}
/**
 End of File
*/